/*

*/


module mxsnandbridge (
  input wb_clk_i,           
  input wb_rst_i,           
  output wbm_cyc_o,          
  output wbm_stb_o,          
  output [15:0] wbm_dat_o,          
  output [31:0] wbm_adr_o,          
  output wbm_we_o,           
  input [15:0] wbm_dat_i,          
  input wbm_ack_i,                  
  output [1:0] wbm_sel_o,          
  
  input [7:0] nand_dat_i,
  output [7:0] nand_dat_o,
  output nand_dat_oe_o,
  input nand_stbn_i,
  input nand_rdnwr_i,
  input nand_csn_i,
  input nand_ale_i,
  
  output nand_rdyn_o
);

reg [2:0] state;
assign state_o = state;
reg [1:0] nand_stb;
reg nand_rdnwr;
reg [1:0] nand_ale;
reg [1:0] nand_cs;
reg nand_dat_oe;	
assign nand_dat_oe_o = nand_dat_oe;
reg [7:0] nand_dat;
assign nand_dat_o = nand_dat;
reg nand_done;
assign nand_done_o = nand_done;
reg wbm_stb;
assign wbm_stb_o = wbm_stb;
reg wbm_cyc;
assign wbm_cyc_o = wbm_cyc;
assign wbm_err_o = 1'b0;
reg nand_err;
assign nand_err_o = nand_err;
assign nand_rdyn_o = nand_err;
reg wbm_we;
assign wbm_we_o = wbm_we;
reg [31:0] wbm_dat;
assign wbm_dat_o = wbm_dat;
reg [15:0] wbm_adr;
assign wbm_adr_o = wbm_adr;
reg [1:0]dstate;
assign dstate_o = dstate;
reg [15:0] dat;
assign wbm_sel_o = 2'b11;

always @(posedge wb_rst_i or posedge wb_clk_i) begin
  if(wb_rst_i) begin
    nand_cs <= 2'b0;
    nand_stb <= 2'b0;
    nand_ale <= 2'b0;
    nand_rdnwr <= 1'b0;
    nand_err <= 1'b0;
    nand_dat_oe <= 1'b0;
    nand_dat <= 8'b0;
    wbm_we <= 1'b0;
    wbm_stb <= 1'b0;
    wbm_cyc <= 1'b0;
    //wbm_err <= 1'b0;
    wbm_dat <= 32'b0;
    wbm_adr <= 16'b0;
    state <= 3'd0;
    dstate <= 2'b0;
    nand_done <= 1'b0;
    dat <= 16'b0;
  end else begin
    nand_cs <= {nand_cs[0], !nand_csn_i};
    nand_stb <= {nand_stb[0], !nand_stbn_i};
    nand_ale <= {nand_ale[0], nand_ale_i};
    nand_rdnwr <= nand_rdnwr_i;
		
    /*if(nand_cs == 2'b11) begin
      nand_err <= wbm_err;
    end*/
    /*if(nand_cs == 2'b00) begin
      state <= 3'd0;
      dstate <= 2'b0;
      wbm_cyc <= 1'b0;
      wbm_stb <= 1'b0;
    end*/
		
    if(wbm_ack_i) begin
      wbm_cyc <= 1'b0;
      wbm_stb <= 1'b0;
      nand_done <= 1'b1;
      dat <= wbm_dat_i;
    end
		
    case(state)
      3'd0: begin
        if((nand_stb == 2'b10) && (nand_ale == 2'b11)) begin
          wbm_adr <= nand_dat_i;
          wbm_we <= nand_rdnwr;
          state <= 3'd1;
        end
        nand_done <= 1'b0;
        nand_dat_oe <= 1'b0;
      end
      3'd1: begin
        nand_dat_oe <= !wbm_we;
        wbm_cyc <= !wbm_we;
        wbm_stb <= !wbm_we;
        nand_err <= 1'b0;
        state <= 3'd2;
        /*if(wbm_we) begin
          nand_dat_oe <= 1'b0;
          state <= 2'd2;
        end else begin
          wbm_cyc <= 1'b1;
          wbm_stb <= 1'b1;
          nand_dat_oe <= 1'b1;
          state <= 3'd2;
        end*/


      end
      3'd2: begin
        if(wbm_we) begin
          if(nand_stb == 2'b10) begin
            wbm_dat <= {wbm_dat[7:0], nand_dat_i};
            dstate <= dstate + 1'b1;
          end	
					
        end else begin
          if(nand_stb == 2'b01) begin
            if(!nand_done) nand_err <= 1'b1;
            nand_dat <= dstate ? dat[7:0] : dat[15:8];
          end  
          if(nand_stb == 2'b10) begin
            dstate <= dstate + 1'b1;
          end
        end


        if(dstate > 2'b1 || nand_err) begin
          if(wbm_we) begin
            wbm_cyc <= 1'b1;
            wbm_stb <= 1'b1;
            state <= 3'd3;
            dstate <= 2'b0;
          end else begin
            if(nand_err) begin
              if(nand_done && (nand_cs == 2'b01)) begin
                nand_err <= 1'b0;
              end else dstate <= 2'b0; 
            end else begin
              dstate <= 2'b0;
              state <= 3'd4;
            end
          end
        end
      end
      3'd3: begin
        nand_err <= 1'b1;
        if(nand_done && (nand_cs == 2'b01)) begin
          nand_err <= 1'b0;
          state <= 3'd4;
        end
      end
      3'd4: begin
        if(!nand_cs[0]) state <= 3'd0;
      end
      default: begin
        state <= 3'd0;
      end
    endcase
  end
end

endmodule
